#! /bin/sh
#
# NET-BASE	A collection of programs that form the base set of the
#		NET-2 Networking Distribution for the LINUX operating
#		system.
#
# Usage:	Install.sh [--devices] [--firsttime] [--nobackup]
#
# Version:	@(#)Install.sh  1.63	01/29/94
#
# Authors:	Fred N. van Kempen, <waltje@uwalt.nl.mugnet.org>
#		Johannes Grosen, <grosen@argv.cs.ndsu.nodak.edu>
#		Copyright 1993,1994 MicroWalt Corporation
#
#		This program is free software; you can redistribute it
#		and/or  modify it under  the terms of  the GNU General
#		Public  License as  published  by  the  Free  Software
#		Foundation;  either  version 2 of the License, or  (at
#		your option) any later version.
#
PATH=/bin:/usr/bin:/sbin:/usr/sbin
export PATH
NROFF="groff -Tascii"	# how to format man pages for catman

BIN_PROGS="hostname netstat"
SBIN_PROGS="arp ifconfig route"
OLDDEVS='net arp route unix inet ip icmp tcp udp lo we* wd* el* ne* hp* sl* plip* dl*'
DEVICES="socket arp route unix inet ip icmp tcp udp lo wd[0-3] el[0-3] ne[0-3] hp[0-3] sl[0-7] plip[0-2] dlink[0-2]"
DEV_OWNER="bin.bin"

  backup()
  {
	if [ "${nobackup}" = "NO" ]; then
		if [ -s $1 ]; then
			mv $1 $1.old
		fi
	fi
  }

  # Display a prompt, and then ask for Y or N.
  yesno() \
  {
	if [ $# = 0 ]; then
		prompt="Are you sure? (y/n): "
	  else
		prompt="$*: "
	fi
	while true
	do
		echo -n ${prompt}
		read answer
		if [ -n "${answer}" -a \
		     "${answer}" = "y" -o "${answer}" = "Y" -o \
		     "${answer}" = "n" -o "${answer}" = "N" ]; then
			break
		fi
		echo "***** WRONG ANSWER!"
	done
	if [ "${answer}" != "y" -a "${answer}" != "Y" ]; then
		return 1
	  else
		return 0
	fi
  }


  devices=NO
  firsttime=NO
  nobackup=NO
  while [ $# != 0 ]; do
	case $1 in
		--devices)
			devices="YES"
			shift
			;;

		--firsttime)
			firsttime="YES"
			shift
			;;

		--nobackup)
			nobackup="YES"
			shift
			;;

		-*)
			echo "Usage: Install.sh [--devices] [--firsttime] [--nobackup]" >&2
			exit 1
			;;

		*)
			shift
			;;
	esac
  done
  echo
  echo "This procedure will install the NET-2 Base Utilities for you."

  ETC="/etc"
  SBIN="/sbin"
  CONF="/etc"
  DIRS="/dev /dev/net /etc /bin /sbin /usr /usr/bin /usr/man"

  # Check out the garbage in /dev.
  DEV="/dev/net"
  if [ -r /dev/net ]; then
	echo ; echo "Checking out your devices..."
	if [ -d /dev/net ]; then
		echo "OK, new-style NET-2e devices already!"
	  else
		echo "Aiii.. old NET-2 devices found!"
	fi

	echo
	if [ ${devices} = "YES" ] && (
		echo "I will nuke all old NET-2 devices for it and"
		yesno "create new ones.  Is that OK? (y/n)"
	); then
		(cd /dev ; /bin/rm -rf ${OLDDEVS})
		if [ ! -d ${DEV} ]; then
			echo ; echo "Creating directory ${DEV}"
			/bin/mkdir ${DEV}
			/bin/chown bin.bin ${DEV}
			/bin/chmod 755 ${DEV}
		fi

		# Create the /dev entries.
  		echo "Creating device special files:"
		for device in ${DEVICES}
		do
			rm -f ${DEV}/${device}
		done

		# Install the SOCKET layer entries.
		NET_MAJOR="16"
		echo -n "socket "
		mknod -m 666 ${DEV}/socket c ${NET_MAJOR} 0
		chown ${DEV_OWNER} ${DEV}/socket
		echo -n "arp "
		mknod -m 600 ${DEV}/arp c ${NET_MAJOR} 1
		chown ${DEV_OWNER} ${DEV}/arp
		echo -n "route "
		mknod -m 600 ${DEV}/route c ${NET_MAJOR} 2
		chown ${DEV_OWNER} ${DEV}/route

		# Install the AF_UNIX layer entries.
		UNIX_MAJOR="17"
		echo -n "unix "
		mknod -m 666 ${DEV}/unix c ${UNIX_MAJOR} 0
		chown ${DEV_OWNER} ${DEV}/unix

		# Install the AF_INET layer entries.
		INET_MAJOR="18"
		echo -n "inet "
		mknod -m 600 ${DEV}/inet c ${INET_MAJOR} 0
		chown ${DEV_OWNER} ${DEV}/inet
		echo -n "ip "
		mknod -m 666 ${DEV}/ip c ${INET_MAJOR} 1
		chown ${DEV_OWNER} ${DEV}/ip
		echo -n "icmp "
		mknod -m 666 ${DEV}/icmp c ${INET_MAJOR} 2
		chown ${DEV_OWNER} ${DEV}/icmp
		echo -n "tcp "
		mknod -m 666 ${DEV}/tcp c ${INET_MAJOR} 3
		chown ${DEV_OWNER} ${DEV}/tcp
		echo -n "udp "
		mknod -m 666 ${DEV}/udp c ${INET_MAJOR} 4
		chown ${DEV_OWNER} ${DEV}/udp

		# Install the LOOPBACK device driver.
		LO_MAJOR="19"
		echo "lo"
		mknod -m 600 ${DEV}/lo c ${LO_MAJOR} 0
		chown ${DEV_OWNER} ${DEV}/lo

		# Install the DP8390 device driver.
		BECKER_MAJOR="20"
		echo -n "wd[0-3] el[0-3] ne[0-3] hp[0-3] "
		for i in 0 1 2 3
		do
			mknod -m 600 ${DEV}/wd$i c ${BECKER_MAJOR} `expr $i + 8`
			chown ${DEV_OWNER} ${DEV}/wd$i
			mknod -m 600 ${DEV}/el$i c ${BECKER_MAJOR} `expr $i + 16`
			chown ${DEV_OWNER} ${DEV}/el$i
			mknod -m 600 ${DEV}/ne$i c ${BECKER_MAJOR} `expr $i + 24`
			chown ${DEV_OWNER} ${DEV}/ne$i
			mknod -m 600 ${DEV}/hp$i c ${BECKER_MAJOR} `expr $i + 32`
			chown ${DEV_OWNER} ${DEV}/hp$i
		done

		# Install the SLIP device driver.
		SLIP_MAJOR="22"
		echo -n "sl[0-7] "
		for i in 0 1 2 3 4 5 6 7
		do
			mknod -m 600 ${DEV}/sl$i c ${SLIP_MAJOR} $i
			chown ${DEV_OWNER} ${DEV}/sl$i
		done

		# Install the DLINK device driver.
		DLINK_MAJOR="26"
		echo -n "dlink[0-2]"
		for i in 0 1 2
		do
			mknod -m 600 ${DEV}/dlink$i c ${DLINK_MAJOR} $i
			chown ${DEV_OWNER} ${DEV}/dlink$i
		done

		# Install the KISS device driver.
		KISS_MAJOR="27"
		echo -n "kiss[0-7]"
		for i in 0 1 2 3 4 5 6 7
		do
			mknod -m 600 ${DEV}/kiss$i c ${KISS_MAJOR} $i
			chown ${DEV_OWNER} ${DEV}/kiss$i
		done

		# Install the DEPCA device driver.
		DEPCA_MAJOR="28"
		echo -n "depca[0-3]"
		for i in 0 1 2 3
		do
			mknod -m 600 ${DEV}/depca$i c ${DEPCA_MAJOR} $i
			chown ${DEV_OWNER} ${DEV}/depca$i
		done

		# Install the PLIP device driver.
		PLIP_MAJOR="29"
		echo -n "plip[0-2] "
		for i in 0 1 2
		do
			mknod -m 600 ${DEV}/plip$i c ${PLIP_MAJOR} $i
			chown ${DEV_OWNER} ${DEV}/plip$i
		done

		echo
	else
		echo
		if [ ${devices} = "NO" ]; then
			if ! yesno "Should I continue anyway? (y/n)"; then
				echo "NET-2 INSTALL ABORTED!"
				exit 1
			fi
		fi
	fi
  fi

  # Are we expected to install preformatted manual pages?
  CATMAN=0
  if [ -d /usr/man/cat1 ]; then
	echo
	if yesno "Do you want me to pre-format the manual pages? (y/n)"; then
		CATMAN=1
	fi
  fi

  # Install the CONFIG files.
  if [ "${firsttime}" = "YES" ]; then
	echo ; echo -n "Installing CONFIG files..."
	cp -a samples/* ${ETC}
	echo "done"
	CONFIG=1
    else
	CONFIG=0
  fi

  # Install the primary user commands.
  echo ; echo "Installing USER commands:"
  for i in ${BIN_PROGS}
  do
	path=/bin/`basename $i`
	echo -n "${path} "
	backup ${path}
	cp $i ${path}
	chmod 511 ${path}
	chown bin.bin ${path}
  done
  (echo -n "/bin/domainname"; cd /bin; ln -sf hostname domainname ; echo)

  # Install the primary system administrator commands.
  echo ; echo "Installing SYSTEM administrator commands:"
  for i in ${SBIN_PROGS}
  do
	path=${SBIN}/`basename $i`
	echo -n "${path} "
	backup ${path}
	cp $i ${path}
	chmod 500 ${path}
	chown bin.bin ${path}
  done
  echo

  # Install the manual pages.
  echo ; echo -n "Installing manual pages..."
  for i in 1 5 8
  do
	# Install manual sources.
	chmod 644 man/*.$i
	chown bin.bin man/*.$i
	cp -af man/*.$i /usr/man/man$i >/dev/null

	# Do we need to pre-format them as well?
	if [ ${CATMAN} -eq 1 ]; then
		for manpage in man/*.$i
		do
			page=`basename ${manpage}`
			rm -f /usr/man/cat${i}/${page}
			${NROFF} -man $manpage > /usr/man/cat${i}/${page}
			chmod 644 /usr/man/cat${i}/${page}
			chown bin.bin /usr/man/cat${i}/${page}
		done
	fi
  done
  (cd /usr/man/man1; ln -sf hostname.1 domainname.1)
  echo

  echo ; echo "Installation completed."
  if [ ${CONFIG} -eq 1 ]; then
	echo
	echo "Do NOT forget to edit the files in ${CONF} just installed."
	echo "or the NET-2 system will not operate correctly!"
  fi
  if [ "${nobackup}" = "NO" ]; then
	echo
	echo "Your old binaries (if they existed) were backed up to <name>.old."
	echo "You may safely delete them once you are satisfied that the new"
	echo "setup is working correctly."
  fi
  echo

  exit 0
